@@ -1,7 +1,7 @@
 package App::SetScreensaverTimeout;
 
-our $DATE = '2014-12-03'; # DATE
-our $VERSION = '0.06'; # VERSION
+our $DATE = '2014-12-08'; # DATE
+our $VERSION = '0.07'; # VERSION
 
 use 5.010001;
 use strict;
@@ -13,6 +13,63 @@ use Proc::Find qw(proc_exists);
 
 our %SPEC;
 
+sub _get_or_set {
+    my ($which, $mins) = @_;
+
+    my $detres = detect_desktop();
+
+    local $Proc::Find::CACHE = 1;
+    if (proc_exists(name=>"gnome-screensaver")) {
+        if ($which eq 'set') {
+            my $secs = $mins*60;
+            system "gsettings", "set", "org.gnome.desktop.session",
+                "idle-delay", $secs;
+            return [500, "gsettings set failed: $!"] if $?;
+        }
+        my $res = `gsettings get org.gnome.desktop.session idle-delay`;
+        return [500, "gsettings get failed: $!"] if $?;
+        $res =~ /^uint32\s+(\d+)$/
+            or return [500, "Can't parse gsettings get output"];
+        return [200, "OK", $1/60, {'func.screensaver'=>'gnome-screensaver'}];
+    }
+
+    if (proc_exists(name=>"xscreensaver")) {
+        my $path = "$ENV{HOME}/.xscreensaver";
+        my $ct = read_file($path);
+        if ($which eq 'set') {
+            my $hours = int($mins/60);
+            $mins -= $hours*60;
+
+            $ct =~ s/^(timeout:\s*)(\S+)/
+                sprintf("%s%d:%02d:%02d",$1,$hours,$mins,0)/em
+                    or return [500, "Can't subtitute timeout setting in $path"];
+            write_file($path, $ct);
+            system "killall", "-HUP", "xscreensaver";
+            $? == 0 or return [500, "Can't kill -HUP xscreensaver"];
+        }
+        $ct =~ /^timeout:\s*(\d+):(\d+):(\d+)\s*$/m
+            or return [500, "Can't get timeout setting in $path"];
+        return [200, "OK", ($1*3600+$2*60+$3)/60,
+                {'func.screensaver'=>'xscreensaver'}];
+    }
+
+    if ($detres->{desktop} eq 'kde-plasma') {
+        my $path = "$ENV{HOME}/.kde/share/config/kscreensaverrc";
+        my $ct = read_file($path);
+        if ($which eq 'set') {
+            my $secs = $mins*60;
+            $ct =~ s/^(Timeout\s*=\s*)(\S+)/${1}$secs/m
+                or return [500, "Can't subtitute Timeout setting in $path"];
+            write_file($path, $ct);
+        }
+        $ct =~ /^Timeout\s*=\s*(\d+)\s*$/m
+            or return [500, "Can't get Timeout setting in $path"];
+        return [200, "OK", $1/60, {'func.screensaver'=>'kde-plasma'}];
+    }
+
+    [412, "Can't detect screensaver type"];
+}
+
 $SPEC{get_screensaver_timeout} = {
     v => 1.1,
     summary => 'Get screensaver timeout (in minutes)',
@@ -28,38 +85,7 @@ _
     },
 };
 sub get_screensaver_timeout {
-    my %args = @_;
-
-    my $detres = detect_desktop();
-
-    if ($detres->{desktop} eq 'kde-plasma') {
-        my $path = "$ENV{HOME}/.kde/share/config/kscreensaverrc";
-        my $ct = read_file($path);
-        $ct =~ /^Timeout\s*=\s*(\d+)\s*$/m
-            or return [500, "Can't get Timeout setting in $path"];
-        return [200, "OK", $1/60, {'func.screensaver'=>'kde-plasma'}];
-    }
-
-    local $Proc::Find::CACHE = 1;
-    if (proc_exists(name=>"gnome-screensaver")) {
-        my $res = `gsettings get org.gnome.desktop.session idle-delay`;
-        return [500, "gsettings get failed: $!"] if $?;
-        $res =~ /^uint32\s+(\d+)$/
-            or return [500, "Can't parse gsettings get output"];
-        return [200, "OK", $1/60, {'func.screensaver'=>'gnome-screensaver'}];
-    }
-
-    if (proc_exists(name=>"xscreensaver")) {
-        my $path = "$ENV{HOME}/.xscreensaver";
-        my $ct = read_file($path);
-
-        $ct =~ /^timeout:\s*(\d+):(\d+):(\d+)\s*$/m
-            or return [500, "Can't get timeout setting in $path"];
-        return [200, "OK", ($1*3600+$2*60+$3)/60,
-                {'func.screensaver'=>'xscreensaver'}];
-    }
-
-    [412, "Can't detect screensaver type"];
+    _get_or_set('get');
 }
 
 my $to_re = '\A\d+(?:\.\d+)?\s*(mins?|minutes?|h|hours?|seconds?|secs?|s)?\z';
@@ -69,10 +95,35 @@ $SPEC{set_screensaver_timeout} = {
     summary => 'Set screensaver timeout',
     description => <<'_',
 
-Provide a common way to quickly set screensaver timeout. Support xscreensaver,
+Provide a common way to quickly set screensaver timeout. Will detect the running
+screensaver/desktop environment and set accordingly. Supports xscreensaver,
 gnome-screensaver, and KDE screen locker. Support for other screensavers will be
 added in the future.
 
+* xscreensaver
+
+  To set timeout for xscreensaver, the program finds this line in
+  `~/.xscreensaver`:
+
+      timeout:    0:05:00
+
+  modifies the line, save the file, and HUP the xscreensaver process.
+
+* gnome-screensaver
+
+  To set timeout for gnome-screensaver, the program executes this command:
+
+      gsettings set org.gnome.desktop.session idle-delay 300
+
+* KDE
+
+  To set timeout for the KDE screen locker, the program looks for this line in
+  `~/.kde/share/config/kscreensaverrc`:
+
+      Timeout=300
+
+  modifies the line, save the file.
+
 _
     args => {
         timeout => {
@@ -109,43 +160,7 @@ sub set_screensaver_timeout {
     $mins = int($mins);
     $mins = 1 if $mins < 1;
 
-    my $detres = detect_desktop();
-
-    if ($detres->{desktop} eq 'kde-plasma') {
-        my $path = "$ENV{HOME}/.kde/share/config/kscreensaverrc";
-        my $ct = read_file($path);
-        my $secs = $mins*60;
-        $ct =~ s/^(Timeout\s*=\s*)(\S+)/${1}$secs/m
-            or return [500, "Can't subtitute Timeout setting in $path"];
-        write_file($path, $ct);
-        return get_screensaver_timeout();
-    }
-
-    local $Proc::Find::CACHE = 1;
-    if (proc_exists(name=>"gnome-screensaver")) {
-        my $secs = $mins*60;
-        system "gsettings", "set", "org.gnome.desktop.session", "idle-delay",
-            $secs;
-        return [500, "gsettings set failed: $!"] if $?;
-        return get_screensaver_timeout();
-    }
-
-    if (proc_exists(name=>"xscreensaver")) {
-        my $path = "$ENV{HOME}/.xscreensaver";
-        my $ct = read_file($path);
-        my $hours = int($mins/60);
-        $mins -= $hours*60;
-
-        $ct =~ s/^(timeout:\s*)(\S+)/
-            sprintf("%s%d:%02d:%02d",$1,$hours,$mins,0)/em
-                or return [500, "Can't subtitute timeout setting in $path"];
-        write_file($path, $ct);
-        system "killall", "-HUP", "xscreensaver";
-        $? == 0 or return [500, "Can't kill -HUP xscreensaver"];
-        return get_screensaver_timeout();
-    }
-
-    [412, "Can't detect screensaver type"];
+    _get_or_set('set', $mins);
 }
 
 1;
@@ -163,7 +178,7 @@ App::SetScreensaverTimeout - Set screensaver timeout
 
 =head1 VERSION
 
-This document describes version 0.06 of App::SetScreensaverTimeout (from Perl distribution App-SetScreensaverTimeout), released on 2014-12-03.
+This document describes version 0.07 of App::SetScreensaverTimeout (from Perl distribution App-SetScreensaverTimeout), released on 2014-12-08.
 
 =head1 FUNCTIONS
 
@@ -195,10 +210,39 @@ Timeout value, in minutes (float)
 
 Set screensaver timeout.
 
-Provide a common way to quickly set screensaver timeout. Support xscreensaver,
+Provide a common way to quickly set screensaver timeout. Will detect the running
+screensaver/desktop environment and set accordingly. Supports xscreensaver,
 gnome-screensaver, and KDE screen locker. Support for other screensavers will be
 added in the future.
 
+=over
+
+=item * xscreensaver
+
+To set timeout for xscreensaver, the program finds this line in
+C<~/.xscreensaver>:
+
+  timeout:    0:05:00
+
+modifies the line, save the file, and HUP the xscreensaver process.
+
+=item * gnome-screensaver
+
+To set timeout for gnome-screensaver, the program executes this command:
+
+  gsettings set org.gnome.desktop.session idle-delay 300
+
+=item * KDE
+
+To set timeout for the KDE screen locker, the program looks for this line in
+C<~/.kde/share/config/kscreensaverrc>:
+
+  Timeout=300
+
+modifies the line, save the file.
+
+=back
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -222,6 +266,20 @@ that contains extra information.
 
  (any)
 
+=head1 KNOWN BUGS
+
+=over
+
+=item * Sometimes fail to lock on KDE
+
+KDE is supposed to pick up on the changes in
+`~/.kde/share/config/kscreensaverrc` immediately, and this is confirmed by
+running the dialog `kcmshell4 screensaver`. However, sometimes the change does
+not take effect and the screensaver won't trigger even after the timeout has
+long passed.
+
+=back
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/App-SetScreensaverTimeout>.
